]> git.lyx.org Git - lyx.git/blobdiff - src/Row.h
Fix invalid iterator access
[lyx.git] / src / Row.h
index 4a1e18ad9afa2ba94da19c35dca61dc51d1bf9fd..13a11b4870e361365bf9a47ae7e049b23f2ed425 100644 (file)
--- a/src/Row.h
+++ b/src/Row.h
@@ -29,6 +29,18 @@ namespace lyx {
 class DocIterator;
 class Inset;
 
+/**
+ * FIXME: Change Row object to operate only on integers and not doubles.
+ *
+ * This use of double is only useful to distribute the extra
+ * horizontal space between separators in justified text. If we do
+ * integer arithmetic, then it is possible to have two groups of
+ * separators, with size s or s+1. Then strings can be drawn without
+ * cutting at separators in justfied text, as it is done in
+ * non-justified text. This will improve performance.
+ */
+
+
 /**
  * An on-screen row of text. A paragraph is broken into a RowList for
  * display. Each Row contains a tokenized description of the contents
@@ -63,16 +75,26 @@ public:
                        : type(t), pos(p), endpos(p + 1), inset(0),
                          extra(0), font(f), change(ch), final(false) {}
 
-               // returns total width of element, including separator overhead
+               // Return total width of element, including separator overhead
                double width() const { return dim.wid + extra; };
-               // returns position in pixels (from the left) of position
-               // \param i in the row element
+               /** Return position in pixels (from the left) of position
+                * \param i in the row element.
+                */
                double pos2x(pos_type const i) const;
-
-               // Return character position that is the closest to
-               // pixel position \param x. The value \param x is
-               // rounded to the actual pixel position.
+               /** Return character position that is the closest to
+                *  pixel position \param x. The value \param x is
+                *  adjusted to the actual pixel position.
+               */
                pos_type x2pos(double &x) const;
+               /** Break the element if possible, so that its width is
+                * less then \param w. Returns true on success.
+                */
+               bool breakAt(double w);
+
+               // Returns the position on left side of the element.
+               pos_type left_pos() const;
+               // Returns the position on right side of the element.
+               pos_type right_pos() const;
 
                // The kind of row element
                Type type;
@@ -125,20 +147,25 @@ public:
                DocIterator const & end) const;
 
        ///
-       void pos(pos_type p);
+       void pos(pos_type p) { pos_ = p; }
        ///
        pos_type pos() const { return pos_; }
        ///
-       void endpos(pos_type p);
+       void endpos(pos_type p) { end_ = p; }
        ///
        pos_type endpos() const { return end_; }
+       ///
+       void right_boundary(bool b) { right_boundary_ = b; }
+       ///
+       bool right_boundary() const { return right_boundary_; }
+
        ///
        Dimension const & dimension() const { return dim_; }
        ///
        Dimension & dimension() { return dim_; }
        ///
        int height() const { return dim_.height(); }
-       ///
+       /// The width of the row, including the left margin, but not the right one.
        int width() const { return dim_.wid; }
        ///
        int ascent() const { return dim_.asc; }
@@ -178,6 +205,10 @@ public:
        ///
        bool empty() const { return elements_.empty(); }
        ///
+       Element & front() { return elements_.front(); }
+       ///
+       Element const & front() const { return elements_.front(); }
+       ///
        Element & back() { return elements_.back(); }
        ///
        Element const & back() const { return elements_.back(); }
@@ -186,11 +217,13 @@ public:
        /// remove all row elements
        void clear() { elements_.clear(); }
        /**
-        * remove all elements after last separator and update endpos
-        * if necessary.
-        * \param keep is the minimum amount of text to keep.
+        * if row width is too large, remove all elements after last
+        * separator and update endpos if necessary. If all that
+        * remains is a large word, cut it to \param width.
+        * \param body_pos minimum amount of text to keep.
+        * \param width maximum width of the row
         */
-       void separate_back(pos_type keep);
+       void shortenIfNeeded(pos_type const body_pos, int const width);
 
        /**
         * If last element of the row is a string, compute its width
@@ -199,10 +232,10 @@ public:
        void finalizeLast();
 
        /**
* Find sequences of RtL elements and reverse them.
         * This should be called once the row is completely built.
         */
-       void reverseRtL();
+       void reverseRTL(bool rtl_par);
 
        friend std::ostream & operator<<(std::ostream & os, Row const & row);
 
@@ -251,6 +284,8 @@ private:
        pos_type pos_;
        /// one behind last pos covered by this row
        pos_type end_;
+       // Is there is a boundary at the end of the row (display inset...)
+       bool right_boundary_;
        /// Row dimension.
        Dimension dim_;
 };
@@ -199,10 +232,10 @@ public:
 
        /**
-        * Find sequences of RtL elements and reverse them.
+        * Find sequences of right-to-left elements and reverse them.
         * This should be called once the row is completely built.
         */